from typing import Dict
from ..utils import path_utils

def form_dotenv_src(__dict: Dict[str, str]):
    env_lines = [f"{key}={value}" for key, value in __dict.items()]
    env_lines.sort()
    return "\n".join(env_lines) if env_lines else ""


def form_python_env_src(env: Dict[str, str], dotenv_path: str = ".env",python_env_path:str="env.py"):

    field_src = lambda key: f"    {key}:Optional[str] = Field(default=None)"
    keys = list(env.keys())
    keys.sort()
    env_fields_src = "\n".join(map(field_src, keys))
    env_init_src = "env = Env(**{" + "key:os.environ.get(key) for key in Env.keys()})\n"

    result = f"""
import os
from typing import Optional
from dotenv import load_dotenv
from pydantic import BaseModel, Field

load_dotenv('{dotenv_path}')

class Env(BaseModel):

    @classmethod
    def keys(cls):
        return cls.__fields__.keys()

{env_fields_src}

{env_init_src}

    """

    if path_utils.path_exists(python_env_path):
        env_src = path_utils.read_text(python_env_path)
        if "env = " in env_src:
            env_src = env_init_src.split(env_init_src,1)[1]
            env_src = "\n".join(env_src.splitlines(keepends=False)[1:])
            result += f"\n{env_src}"
    return result


def form_dockerfile_env_src(
    __dict: Dict[str, str], dockerfile_path: str = "Dockerfile"
):
    dockerfile_lines = []
    if path_utils.path_exists(dockerfile_path):
        dockerfile_env_src = path_utils.read_text(dockerfile_path)
        dockerfile_lines = dockerfile_env_src.splitlines(keepends=False)
    dockerfile_lines = [line for line in dockerfile_lines if not line.startswith("ENV")]
    endline = ""
    if dockerfile_lines:
        endline = dockerfile_lines[-1]
        dockerfile_lines = dockerfile_lines[:-1]
    env_lines = [f"ENV {key}={value}" for key, value in __dict.items()]
    result = dockerfile_lines + env_lines + [endline]
    return "\n".join(result)



from typing import List
from enum import Enum
from . import env_lines as __env_lines
from ..utils import path_utils
def __env_dict_from_lines(__env_lines: List[str]):
    env_lines = __env_lines.copy()
    env_lines.sort()
    env_kv_pairs = [line.split("=", 1) for line in env_lines if "=" in line]
    return {key: value for (key, value) in env_kv_pairs}



def __env_dict_from_lines(__env_lines: List[str]):
    env_lines = __env_lines.copy()
    env_lines.sort()
    env_kv_pairs = [line.split("=", 1) for line in env_lines if "=" in line]
    return {key: value for (key, value) in env_kv_pairs}



def docker_compose_dict(__docker_compose_src: str):
    return __env_dict_from_lines(__env_lines.docker_compose_lines(__docker_compose_src))


def dockerfile_dict(__src: str):
    return __env_dict_from_lines(__env_lines.dockerfile_lines(__src))


def dotenv_dict(__dotenv_src: str):
    return __env_dict_from_lines(__env_lines.dotenv_lines(__dotenv_src))




class EnvType(Enum):

    DOTENV = 1
    DOCKERFILE = 2
    DOCKER_COMPOSE = 3
    NONE = 4



def env_dict(path: str = None, src: str = None, env: EnvType = EnvType.NONE):
    if src is not None:
        if env == EnvType.DOCKER_COMPOSE:
            result = docker_compose_dict(src)
        elif env == EnvType.DOCKERFILE:
            result = dockerfile_dict(src)
        elif env == EnvType.DOTENV:
            result = dotenv_dict(src)
        else:
            raise Exception(f"unrecognized env_type: {str(env)}")

    else:
        if path is None:
            raise Exception("no param for env_dict")
        else:
            src = path_utils.read_text(path)
        result = env_dict(src=src, env=env)
    return result




def __safe_merge(__lhs:dict,__rhs:dict):
    result = __lhs.copy()
    for key,value in __rhs.items():
        if key in __lhs:
            lvalue = __lhs.get(key)
            if lvalue != value:
                raise Exception(f"ERROR: CONFLICT OF DICT KEYS:: {str(key)} : {lvalue} : {value}")
        result[key] = value
    return result


def safe_merge(__lhs:dict,__rhs:dict):
    return __safe_merge(__lhs,__rhs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dockertk', 'dockertk.core', 'dockertk.utils']

package_data = \
{'': ['*']}

install_requires = \
['pytest>=6.1,<7.0']

setup_kwargs = {
    'name': 'dockertk',
    'version': '1.0.15',
    'description': '',
    'long_description': '',
    'author': 'wayfaring-stranger',
    'author_email': 'zw6p226m@duck.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from .core import env_dict
from .core import env_src
from .utils import dict_utils
from .utils import path_utils

def docker_compose_dict(path: str):
    return env_dict.env_dict(path=path, env=env_dict.EnvType.DOCKER_COMPOSE)

def dotenv_dict(path: str):
    return env_dict.env_dict(path=path, env=env_dict.EnvType.DOTENV)

def dockerfile_dict(path: str):
    return env_dict.env_dict(path=path, env=env_dict.EnvType.DOCKERFILE)


from dataclasses import dataclass

@dataclass(frozen=False,order=True)
class EnvPaths:
    docker_compose_path: str
    dotenv_path: str
    dockerfile_path: str
    python_env_path: str



def unified_environment(env_paths:EnvPaths):
    result = {}
    path = env_paths.docker_compose_path
    if path_utils.path_exists(path):
        docker_compose_env = docker_compose_dict(path=path)
        result = dict_utils.safe_merge(result,docker_compose_env)
    path = env_paths.dockerfile_path
    if path_utils.path_exists(path):
        dockerfile_env = dockerfile_dict(path)
        result = dict_utils.safe_merge(result,dockerfile_env)
    path = env_paths.dotenv_path
    if path_utils.path_exists(path):
        dotenv_env = dotenv_dict(path)
        result = dict_utils.safe_merge(result,dotenv_env)
    return result

def refresh_envs(
    docker_compose_path: str = "docker-compose.yml",
    dotenv_path: str = ".env",
    dockerfile_path: str = "Dockerfile",
    python_env_path: str = "env.py",
):

    env_paths = EnvPaths(
        **dict(
            docker_compose_path=docker_compose_path,
            dotenv_path=dotenv_path,
            dockerfile_path=dockerfile_path,
            python_env_path=python_env_path
        )
    )
    env = unified_environment(env_paths=env_paths)

    if env:
        dotenv_src = env_src.form_dotenv_src(env)
        python_src = env_src.form_python_env_src(env=env, dotenv_path=dotenv_path,python_env_path=python_env_path)
        dockerfile_src = env_src.form_dockerfile_env_src(env, dockerfile_path)
        path_utils.write_text(dotenv_path, dotenv_src)
        path_utils.write_text(python_env_path, python_src)
        path_utils.write_text(dockerfile_path, dockerfile_src)
